/* crypto/aes/aes_cbc.c -*- mode:C; c-file-style: "eay" -*- */
/* ====================================================================
 * Copyright (c) 1998-2002 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#ifndef AES_DEBUG
# ifndef NDEBUG
#  define NDEBUG
# endif
#endif
#include <assert.h>

#include <openssl/aes.h>
#include "aes_locl.h"

extern AES_OPENSSL_CALLBACK_FUNCS aes_openssl_callbacks;

/*ad*
 * Encrypts/decrypts data using the Cipher Block Chaining AES 
 * algorithm. key->round value will decide whether it is AES128, AES192, or
 * AES256 CBC. 
 *
 * @external
 * @category Cryptography:AES
 * @since 6.2
 * @see @url http://www.ietf.org/rfc/rfc3602.txt AES-CBC:RFC 3602
 * @include <aesapi.h> 
 *
 * @param in      Input data
 * @param out     Encrypted output data
 * @param length  Input data length in bytes
 * @param key     Encryption key schedule  
 * @param ivec    Initialization vector
 * @param enc     AES_ENCRYPT or AES_DECRYPT 
 *
 * @note Sample code usage
 *
 * <example>
 * 	#include <stdio.h>
 * 	#include <stdlib.h>
 * 	#include <aesapi.h>
 *
 * 	void aes_cbc_test(char *key, char *pt, char *iv, char *ct)
 * 	{
 *    AES_KEY a_key;
 *    unsigned char iOut[AES_BLOCK_SIZE]; 
 *    unsigned char iIn[AES_BLOCK_SIZE]; 
 *
 *    if (AES_set_encrypt_key(key, 256, &a_key))
 *    {
 *            printf("\nAES_set_encrypt_key error\n");
 *    }
 *    AES_cbc_encrypt(pt, iOut, (unsigned long)AES_BLOCK_SIZE, (const AES_KEY *)&a_key, iv, AES_ENCRYPT);
 *       
 *    if(memcmp(iOut, ct, sizeof iOut))
 *    {
 *            printf("\nAES_cbc_encrypt AES_ENCRYPT error\n");
 *    }
 *
 *    if (AES_set_decrypt_key(key, 256, &a_key))
 *    {
 *            printf("\nAES_set_decrypt_key error\n");
 *    }
 *    AES_cbc_encrypt(iOut, iIn, (unsigned long)AES_BLOCK_SIZE, (const AES_KEY *)&a_key, tmp, AES_DECRYPT);
 *
 *    if(memcmp(iIn, pt, sizeof iIn))
 *    {
 *        printf("\nAES_cbc_encrypt AES_DECRYPT error\n");    
 *    }
 * 	}
 *
 * </example>
*ad*/
void AES_cbc_encrypt(const unsigned char * in, unsigned char * out, const unsigned long length, const AES_KEY * key, unsigned char * ivec, const int enc) 
{
	unsigned long n;
	unsigned long len = length;
	unsigned char tmp[AES_BLOCK_SIZE];

	assert(in && out && key && ivec);
	assert((AES_ENCRYPT == enc)||(AES_DECRYPT == enc));

        if (aes_openssl_callbacks.aesCbcEncrypt == NULL)
	{
	if (AES_ENCRYPT == enc) {
		while (len >= AES_BLOCK_SIZE) {
			for(n=0; n < sizeof tmp; ++n)
				tmp[n] = in[n] ^ ivec[n];
			AES_encrypt(tmp, out, key);
			memcpy(ivec, out, AES_BLOCK_SIZE);
			len -= AES_BLOCK_SIZE;
			in += AES_BLOCK_SIZE;
			out += AES_BLOCK_SIZE;
		}
		if (len) {
			for(n=0; n < len; ++n)
				tmp[n] = in[n] ^ ivec[n];
			for(n=len; n < AES_BLOCK_SIZE; ++n)
				tmp[n] = ivec[n];
			AES_encrypt(tmp, tmp, key);
			memcpy(out, tmp, len);
			memcpy(ivec, tmp, sizeof tmp);
		}		
	} else {
		while (len >= AES_BLOCK_SIZE) {
			memcpy(tmp, in, sizeof tmp);
			AES_decrypt(in, out, key);
			for(n=0; n < AES_BLOCK_SIZE; ++n)
				out[n] ^= ivec[n];
			memcpy(ivec, tmp, AES_BLOCK_SIZE);
			len -= AES_BLOCK_SIZE;
			in += AES_BLOCK_SIZE;
			out += AES_BLOCK_SIZE;
		}
		if (len) {
			memcpy(tmp, in, sizeof tmp);
			AES_decrypt(tmp, tmp, key);
			for(n=0; n < len; ++n)
				out[n] ^= ivec[n];
			memcpy(ivec, tmp, sizeof tmp);
		}
	}
	}
	else /* aes_openssl_callbacks.aesCbcEncrypt is non-NULL */
	{
	    aes_openssl_callbacks.aesCbcEncrypt(in, out, length, key, ivec, enc);

            /* init vector must be the last ciphertext processed */
            if (AES_ENCRYPT == enc)
            {
                memcpy(ivec, &out[length - AES_BLOCK_SIZE], AES_BLOCK_SIZE);
            }
            else
            {
	        memcpy(ivec, &in[length - AES_BLOCK_SIZE], AES_BLOCK_SIZE);
            }
	}
}


